; setup our Screen Width and Height here for easier tracking
#ScreenWidth  = 640
#ScreenHeight = 480

; Initialize the sprite and keyboard systems and a 640x480, 16-bit screen
If InitSprite() = 0 Or InitKeyboard() = 0 Or OpenScreen(#ScreenWidth,#ScreenHeight,16,"App Title") = 0
   MessageRequester("Error!", "Unable to Initialize Environment", #PB_MessageRequester_Ok)
   End
EndIf

ClearColor = RGB(0,0,0) 

; Use the PNG decoder for image loads.
UsePNGImageDecoder()

; load in the sprite
SpriteID = LoadSprite(#PB_Any,"ball.png")

; get the width and height of the sprites
Sprite_Width  = SpriteWidth(SpriteID)
Sprite_Height = SpriteHeight(SpriteID)

Structure Balls
    X.w     ; track the X position of the ball
    Y.w     ; track the Y position of the ball
    XDir.b  ; track the X direction of the ball (0=left,1=right)
    YDir.b  ; track the Y direction of the ball (0=up,  1=down)
    Speed.b ; track the speed of the ball
EndStructure 

; setup a list of balls
NewList Ball.Balls()

; create a bunch of instances of balls
For BCounter = 0 To 10
    If AddElement(Ball()) <> 0
       Ball()\X = Random(#ScreenWidth - Sprite_Width)
       Ball()\Y = Random(#ScreenHeight - Sprite_Height)
       Ball()\XDir = Random(1)
       Ball()\YDir = Random(1)
       Ball()\Speed = Random(2) + 1
    Else
       MessageRequester("Error!", "Unable to allocate memory for new element", #PB_MessageRequester_Ok)
       End
    EndIf
Next

Hits.w = 0

Repeat
   ; clear the screen
   ClearScreen(ClearColor)

   ForEach Ball()

      ; show our sprite (using transparency)
      DisplayTransparentSprite(SpriteID,Ball()\X,Ball()\Y)

      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ; First let's work with the X-Axis
      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ; if we're moving left, subtract speed
      If Ball()\XDir = 0
         Ball()\X = Ball()\X - Ball()\Speed
      Else
         ; we must be moving right, add speed
         Ball()\X = Ball()\X + Ball()\Speed
      EndIf

      ; if we hit the right edge, start moving left
      If Ball()\X > #ScreenWidth - Sprite_Width
         Ball()\XDir = 0
      EndIf
      ; if we hit the left edge, start moving right
      If Ball()\X < 1
         Ball()\XDir = 1
      EndIf

      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ; Now let's work with the Y-Axis
      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ; if we're moving up, subtract speed
      If Ball()\YDir = 0
         Ball()\Y = Ball()\Y - Ball()\Speed
      Else
         ; we must be moving down, add speed
         Ball()\Y = Ball()\Y + Ball()\Speed
      EndIf

      ; if we hit the bottom edge, start moving up
      If Ball()\Y > #ScreenHeight - Sprite_Height
         Ball()\YDir = 0
      EndIf
      ; if we hit the top edge, start moving down
      If Ball()\Y < 1
         Ball()\YDir = 1
      EndIf
      
      ; first let's save the current instance of BALL()
      CurrentBall = Ball()
      ; then let's store the X & Y values for the current ball
      CurrentBallX = Ball()\X
      CurrentBallY = Ball()\Y
      
      ; run through the list of balls
      ForEach Ball()
         ; make sure we're not on the current ball
         If Ball() <> CurrentBall
            ; do a bounding box collision check
            If SpriteCollision(SpriteID,Ball()\X,Ball()\Y,SpriteID,CurrentBallX,CurrentBallY)
               ; we have an overlap, so add one to our variable
               Hits = Hits + 1
            EndIf
         EndIf
      Next
      ; set the Ball() pointer back to the current ball
      ChangeCurrentElement(Ball(),CurrentBall)
   Next   

   ; put up text for exiting
   StartDrawing(ScreenOutput())
     DrawText(0,0,"Hits: " + Str(Hits))
     DrawText(0,460,"Press any key To quit")
   StopDrawing()

   FlipBuffers()

   ExamineKeyboard()
Until KeyboardReleased(#PB_Key_All)

End



; IDE Options = PureBasic 4.61 (Windows - x64)
; CursorPosition = 16
; FirstLine = 24