; setup our Screen Width and Height here for easier tracking
#ScreenWidth = 640
#ScreenHeight = 480

; Initialize the sprite and keyboard, and a 640x480, 16-bit screen, add #PB_Screen_NoSynchronization so we don't use syncing
If InitSprite() = 0 Or InitKeyboard() = 0 Or OpenScreen(#ScreenWidth,#ScreenHeight,16,"App Title",#PB_Screen_NoSynchronization) = 0
   MessageRequester("Error!", "Unable to Initialize Environment", #PB_MessageRequester_Ok)
   End
EndIf

ClearColor = RGB(0,0,0)

; setup our structure for the boxes
Structure BoxStructure
  X.f : Y.f 
  Width.l : Height.l
  r.l : g.l : b.l
  XUnits.f : YUnits.f
EndStructure

; make sure we have a list for the boxes
Global NewList Boxes.BoxStructure()

 ;****************************************************
 ;  Procedure: CreateBoxes()
 ;     Author: John Logsdon & Derlidio Siqueira
 ;    Purpose: Creates a new box with random info
 ;       Args: Amount - Number of boxes to create
 ;****************************************************
Procedure CreateBoxes(Amount.l)
  For i = 0 To (Amount - 1)
    If AddElement(Boxes()) <> 0
      Boxes()\X = 0 
      Boxes()\Y = 0  
      Boxes()\r = Random(254)
      Boxes()\g = Random(254)
      Boxes()\b = Random(254)       
      Boxes()\Width = Random(25) + 5
      Boxes()\Height = Random(25) + 5
      Boxes()\XUnits = ((Random(1000) * 0.375)/1000.0 + 0.050)  
      Boxes()\YUnits = ((Random(1000) * 0.375)/1000.0 + 0.050)
    Else
      MessageRequester("Error!", "Unable to add element", #PB_MessageRequester_Ok)
      End
    EndIf
  Next
EndProcedure

 ;****************************************************
 ;  Procedure: UpdateBoxes()
 ;     Author: John Logsdon & Derlidio Siqueira
 ;    Purpose: Moves our boxes around the screen
 ;       Args: Modifier - The time since last call
 ;****************************************************
Procedure UpdateBoxes(Modifier.f)
  ForEach(Boxes())
    ; Add X's current value to the number of units it is to move
    ; multiplied the amount of time that has passed
    Boxes()\X = Boxes()\X + (Boxes()\XUnits * Modifier)
    ; if the resultant value goes beyond the screen, reset to 0
    If Boxes()\X > #ScreenWidth
       Boxes()\X = 0
    EndIf
    
    ; Add Y's current value to the number of units it is to move
    ; multiplied the amount of time that has passed
    Boxes()\Y = Boxes()\Y + (Boxes()\YUnits * Modifier)
    ; if the resultant value goes beyond the screen, reset to 0
     If Boxes()\Y > #ScreenHeight
        Boxes()\Y = 0
     EndIf
    
    If StartDrawing(ScreenOutput()) 
       ; draw the box
      Box(Boxes()\X,Boxes()\Y,Boxes()\Width,Boxes()\Height,RGB(Boxes()\r,Boxes()\g,Boxes()\b))
    Else
      MessageRequester("Error!", "Unable to Draw to ScreenOutput()", #PB_MessageRequester_Ok)
      End
    EndIf
    
    StopDrawing()
  Next
EndProcedure

; setup our timers and init their value
FPSTimer = ElapsedMilliseconds()
Start_Time.f = ElapsedMilliseconds()

; setup our FPS tracking variables
FPS = 0
FPSCounter = 0

; create some boxes
CreateBoxes(100)

; Keep going until the user hits ESCAPE
Repeat
   ; clear the screen
  ClearScreen(ClearColor)
  ; get the current time
  Current_Time.f = ElapsedMilliseconds()
  ; determine how much time has passed
  Time_Passed.f = Current_Time.f - Start_Time.f
  ; reset the start time
  Start_Time.f = Current_Time.f

  ; update and draw the boxes to the screen
  UpdateBoxes(Time_Passed.f)
  
  If StartDrawing(ScreenOutput()) 
    ; show the current FPS
    DrawText(0,460,"FPS = " + Str(FPS) + "  --  Press ESC to exit...")
  Else
    MessageRequester("Error!", "Unable to Draw to ScreenOutput()", #PB_MessageRequester_Ok)
    End
  EndIf
  
  StopDrawing()
  
  ExamineKeyboard()

  FlipBuffers()
  
  ; if the current time is 1000 milliseconds (1 second)
  ; past the starting timer
  If ElapsedMilliseconds() > FPSTimer + 1000
    ; set the FPS var to the FPS counter
    FPS = FPSCounter
    ; reset the counter
    FPSCounter = 0
    ; reset the timer
    FPSTimer = ElapsedMilliseconds()
  Else
    ; otherwise, ad 1 to the counter variable
    FPSCounter = FPSCounter + 1
  EndIf
  
Until KeyboardReleased(#PB_Key_Escape)

End
; IDE Options = PureBasic 4.61 (Windows - x64)
; CursorPosition = 27
; FirstLine = 14
; Folding = -