;***************************************************************************
;*   Library Name: Health
;*       Coded By: John Logsdon
;*   Last Updated: 8/5/2012
;*        Purpose: Handles all of the health pak procedures
;***************************************************************************


;*************************************************************
; PROCEDURE: AddHealthPak(...)
;
; This function adds a healthpak to the HealthPak() list.
;
; Arguments: 
;   X: X position (array position, not pixel)
;   Y: Y position (array position, not pixel)
;   TileWidth: how wide are the images
;   TileHeight: how high are the images
;
; Returns:
;   N/A
;*************************************************************
Procedure AddHealthPak(X.l,Y.l,TileWidth,TileHeight)
  X = (X * TileWidth) + 8
  Y = (Y * TileHeight) + 16
  ; Add it to the ladder Type
  AddElement(HealthPak())
  HealthPak()\WorldX = X
  HealthPak()\WorldY = Y
  HealthPak()\ScreenX = X
  HealthPak()\ScreenY = Y
EndProcedure


Procedure MoveHealthPaks(Current_Time,MoveMap.l,Direction.l,TileWidth,TileHeight)
  ; go through the healthpaks
  ForEach(HealthPak())
     ; move them in relation to the movement of the map
     If MoveMap.l = 1
        Select Direction.l 
           Case 1
               HealthPak()\ScreenY = HealthPak()\ScreenY + #MoveDistance
           Case 2
               HealthPak()\ScreenY = HealthPak()\ScreenY - #MoveDistance
           Case 3
               HealthPak()\ScreenX = HealthPak()\ScreenX + #MoveDistance
           Case 4
               HealthPak()\ScreenX = HealthPak()\ScreenX - #MoveDistance
        EndSelect
     EndIf

     ; see if the player has hit a pak, and if the player NEEDS the pak or not
     If PlayerArmor < 100 And SpritePixelCollision(Player(PlayerFrame)\Image,PlayerWorldX,PlayerWorldY, HealthPak_Image,HealthPak()\WorldX,HealthPak()\WorldY)
        ; player obviously needs the pak, so give them a boost
        PlayerArmor = PlayerArmor + 10
        ; push up their healthbar a bit
        PlayerHealthBar = PlayerHealthBar + 3
        ; play a happy sound
        MyPlaySound(11)
        ; and nuke the healthpak
        DeleteElement(HealthPak())
     Else
        ; otherwise, just show it
        DisplayTransparentSprite(HealthPak_Image,HealthPak()\ScreenX,HealthPak()\ScreenY)
     EndIf
  Next
EndProcedure

;*************************************************************
; PROCEDURE: DeleteHealthPaks(...)
;
; Nukes all the health paks from the list
;
;*************************************************************
Procedure DeleteHealthPaks()
  ForEach(HealthPak())
     DeleteElement(HealthPak())
  Next
EndProcedure 
; IDE Options = PureBasic 4.61 (Windows - x64)
; CursorPosition = 69
; FirstLine = 33
; Folding = -