;***************************************************************************
;*   Library Name: Levels
;*       Coded By: John Logsdon
;*   Last Updated: 8/11/2012
;*        Purpose: Handles the levels
;***************************************************************************


;*************************************************************
; PROCEDURE: SetupCollisionPoints()
;
; Call the Map functions to add walls/ladders to the collision
; list.
;
; Returns: n/a
;*************************************************************
Procedure SetupCollisionPoints(TileWidth,TileHeight)
  ; First let's set up the walls
  Map_AddWall(0,TileWidth,TileHeight)
  Map_AddWall(1,TileWidth,TileHeight)
  Map_AddWall(2,TileWidth,TileHeight)
  Map_AddWall(3,TileWidth,TileHeight)
  Map_AddWall(4,TileWidth,TileHeight)
  Map_AddWall(5,TileWidth,TileHeight)
  Map_AddWall(6,TileWidth,TileHeight)
  Map_AddWall(7,TileWidth,TileHeight)
  Map_AddWall(8,TileWidth,TileHeight)
  Map_AddWall(9,TileWidth,TileHeight)
  Map_AddWall(10,TileWidth,TileHeight)
  Map_AddWall(11,TileWidth,TileHeight)
  Map_AddWall(12,TileWidth,TileHeight)
  Map_AddWall(13,TileWidth,TileHeight)
  Map_AddWall(14,TileWidth,TileHeight)
  Map_AddWall(15,TileWidth,TileHeight)
  Map_AddWall(16,TileWidth,TileHeight)
  Map_AddWall(17,TileWidth,TileHeight)
  Map_AddWall(18,TileWidth,TileHeight)    
  Map_AddWall(33,TileWidth,TileHeight)    
  Map_AddWall(34,TileWidth,TileHeight)    
  Map_AddWall(35,TileWidth,TileHeight)    
  Map_AddWall(36,TileWidth,TileHeight)    
  Map_AddWall(37,TileWidth,TileHeight)    
  Map_AddWall(38,TileWidth,TileHeight)    
  Map_AddWall(39,TileWidth,TileHeight)    
  Map_AddWall(40,TileWidth,TileHeight)    
  Map_AddWall(41,TileWidth,TileHeight)    
  Map_AddWall(42,TileWidth,TileHeight)    
  Map_AddWall(49,TileWidth,TileHeight)    
  
  ; Then we'll add in our ladders
  Map_AddLadder(43,TileWidth,TileHeight)    
  Map_AddLadder(44,TileWidth,TileHeight)    
  
EndProcedure

 
;*************************************************************
; PROCEDURE: StartLevel(...)
;
; This function clears out all old data and starts a fresh level.
;
; Arguments: 
;   Current_Level: what level to start?
;   TileWidth: how wide are the tiles
;   TileHeight: how high are the tiles
;
; Returns:
;   N/A
;*************************************************************
Procedure StartLevel(Current_Level,TileWidth,TileHeight)

  ; reset all the coordinates
  ScreenX = ViewPortWidth / 2 
  ScreenY = ViewPortHeight / 2
  ScreenCenterX = ScreenX
  ScreenCenterY = ScreenY
  Current_Time = ElapsedMilliseconds()
  StartingLevel = 1
  StartingLevelTime = 6
  
  ; let's make sure we're not in drawing-mode
  StopDrawing();
  
  ; see what level was selected
  Select Current_Level
     Case 1
          ; Load the tiles for the Map
          Map_LoadTiles("graphics\migztiles.png",TileWidth, TileHeight,0)
        
          ; Reset the map
          Map_ResetMap(0,0)
        
          ; Load the actual map
          Map_LoadBinaryMap("migzlevel1.dat")
        
          ; Setup the wall objects so we know where we can expect to collide
          SetupCollisionPoints(TileWidth,TileHeight)
        
          ; Initialize the Player's bounding boxes
          Map_SetupPlayerBoundingBoxes(7,0,24,1, 7,30,24,31, 7,0,8,31, 23,0,24,31)
        
          ; reset the pixel width's and height's accordingly
          Map_Pixel_Width= Map_Width * TileWidth 
          Map_Pixel_Height = Map_Height * TileHeight
        
          ; nuke all previous robots
          DeleteRobots()
          
          ; nuke all previous lasers
          DeleteLasers();
          
          ; nuke all previous healthpaks
          DeleteHealthPaks()
        
          ; reset the player information
          PlayerWorldX = 2 * TileWidth  ; start point X
          PlayerWorldY = 6 * TileHeight ; start point Y
          PlayerExitX = 37 * TileWidth  ; exit point X
          PlayerExitY = 3 * TileHeight  ; exit point Y
          DrawPlayerX = ScreenCenterX
          DrawPlayerY = ScreenCenterY
          PlayerFrame = 0
          PlayerDead = 0
          PlayerHit = 0
          PlayerHitAnimation = 0
          PlayerFacing = 4
          PlayerArmor = 100
          PlayerHealthBar = 30
          Fidget_Level = 0
          Fidget_Timer = Current_Time
          PlayerFidgeting = 0
          Fidget_Interval = 5000
          Fidget_Speed = 250
          PlayerSleeping = 0
          PlayerFiring = 0
          LaserSpeed = 8
          RobotArmorHit = 7
          PlayerArmorHit = 10
          AllowableDistance = 30  
          RHits = 3
          
          ; Just pick spots on the map (the X,Y positions of the map array) that look good for robots to hang out
          AddRobot(15,6,1,26,TileWidth,TileHeight)
          AddRobot(26,6,1,26,TileWidth,TileHeight)
          AddRobot(33,6,28,34,TileWidth,TileHeight)
          AddRobot(6,13,1,15,TileWidth,TileHeight)
          AddRobot(2,17,1,30,TileWidth,TileHeight)
          AddRobot(22,17,1,30,TileWidth,TileHeight)
          AddRobot(35,17,32,38,TileWidth,TileHeight)
          AddRobot(28,15,25,30,TileWidth,TileHeight)
          AddRobot(30,12,25,31,TileWidth,TileHeight)
          AddRobot(25,9,25,34,TileWidth,TileHeight)

          ; Just pick spots on the map (the X,Y positions of the map array) that look good for healthpaks to be
          AddHealthPak(4,1,TileWidth,TileHeight)
          AddHealthPak(26,6,TileWidth,TileHeight)
          AddHealthPak(3,9,TileWidth,TileHeight)
          AddHealthPak(25,9,TileWidth,TileHeight)
          AddHealthPak(31,12,TileWidth,TileHeight)
          AddHealthPak(15,13,TileWidth,TileHeight)
          AddHealthPak(2,17,TileWidth,TileHeight)
          AddHealthPak(29,17,TileWidth,TileHeight)
          AddHealthPak(34,6,TileWidth,TileHeight)
          AddHealthPak(33,12,TileWidth,TileHeight)

     Case 2 ;;;; See the comments in Case 1 for details
          Map_LoadTiles("graphics\migztiles.png",TileWidth, TileHeight,0)
          Map_ResetMap(0,0)
          Map_LoadBinaryMap("migzlevel2.dat")
          SetupCollisionPoints(TileWidth,TileHeight)
          Map_SetupPlayerBoundingBoxes(7,0,24,1, 7,30,24,31, 7,0,8,31, 23,0,24,31)

          Map_Pixel_Width= Map_Width * TileWidth 
          Map_Pixel_Height = Map_Height * TileHeight
        
          ForEach(Robot())
             DeleteElement(Robot())
          Next
          ForEach(Laser())
             DeleteElement(Laser())
          Next
          ForEach(HealthPak())
             DeleteElement(HealthPak())
          Next
        
          PlayerWorldX = 1 * TileWidth
          PlayerWorldY = 1 * TileHeight
          PlayerExitX = 38 * TileWidth
          PlayerExitY = 17 * TileHeight
          DrawPlayerX = ScreenCenterX
          DrawPlayerY = ScreenCenterY
        
          PlayerFrame = 0
          PlayerDead = 0
          PlayerHit = 0
          PlayerHitAnimation = 0
          PlayerFacing = 4
          PlayerArmor = 100
          PlayerHealthBar = 30
          Fidget_Level = 0
          Fidget_Timer = Current_Time
          PlayerFidgeting = 0
          Fidget_Interval = 5000
          Fidget_Speed = 250
          PlayerSleeping = 0
          PlayerFiring = 0
          LaserSpeed = 8
          RobotArmorHit = 6
          PlayerArmorHit = 10
          AllowableDistance = 30  
          RHits = 4
          
          AddRobot(6,1,3,21,TileWidth,TileHeight)
          AddRobot(18,1,3,21,TileWidth,TileHeight)
          AddRobot(10,9,1,15,TileWidth,TileHeight)
          AddRobot(4,6,3,22,TileWidth,TileHeight)
          AddRobot(21,6,3,22,TileWidth,TileHeight)
          AddRobot(2,13,1,15,TileWidth,TileHeight)
          AddRobot(10,17,1,30,TileWidth,TileHeight)
          AddRobot(16,17,1,30,TileWidth,TileHeight)
          AddRobot(24,17,1,30,TileWidth,TileHeight)
          AddRobot(34,8,27,34,TileWidth,TileHeight)
          AddRobot(26,14,20,30,TileWidth,TileHeight)
          AddRobot(20,11,20,34,TileWidth,TileHeight)
          AddRobot(29,11,20,34,TileWidth,TileHeight)
          AddRobot(28,5,27,38,TileWidth,TileHeight)
          AddRobot(38,5,27,38,TileWidth,TileHeight)

          AddHealthPak(1,1,TileWidth,TileHeight)
          AddHealthPak(13,1,TileWidth,TileHeight)
          AddHealthPak(21,1,TileWidth,TileHeight)
          AddHealthPak(8,13,TileWidth,TileHeight)
          AddHealthPak(1,17,TileWidth,TileHeight)
          AddHealthPak(13,17,TileWidth,TileHeight)
          AddHealthPak(27,17,TileWidth,TileHeight)
          AddHealthPak(32,17,TileWidth,TileHeight)
          AddHealthPak(34,17,TileWidth,TileHeight)
          AddHealthPak(22,6,TileWidth,TileHeight)
          AddHealthPak(25,17,TileWidth,TileHeight)
          AddHealthPak(22,14,TileWidth,TileHeight)
          AddHealthPak(20,11,TileWidth,TileHeight)
          AddHealthPak(27,11,TileWidth,TileHeight)
          AddHealthPak(34,8,TileWidth,TileHeight)
          AddHealthPak(14,9,TileWidth,TileHeight)
     

     Case 3 ;;;; See the comments in Case 1 for details
          Map_LoadTiles("graphics\migztiles.png",TileWidth, TileHeight,0)
        
          Map_ResetMap(0,0)

          Map_LoadBinaryMap("migzlevel3.dat")

          SetupCollisionPoints(TileWidth,TileHeight)
        
          Map_SetupPlayerBoundingBoxes(7,0,24,1, 7,30,24,31, 7,0,8,31, 23,0,24,31)

          Map_Pixel_Width= Map_Width * TileWidth 
          Map_Pixel_Height = Map_Height * TileHeight
        
          ForEach(Robot())
             DeleteElement(Robot())
          Next
          ForEach(Laser())
             DeleteElement(Laser())
          Next
          ForEach(HealthPak())
             DeleteElement(HealthPak())
          Next
        
          PlayerWorldX = 11 * TileWidth
          PlayerWorldY = 7 * TileHeight
          PlayerExitX = 1 * TileWidth
          PlayerExitY = 37 * TileHeight
          DrawPlayerX = ScreenCenterX
          DrawPlayerY = ScreenCenterY
        
          PlayerFrame = 0
          PlayerDead = 0
          PlayerHit = 0
          PlayerHitAnimation = 0
          PlayerFacing = 4
          PlayerArmor = 100
          PlayerHealthBar = 30
          Fidget_Level = 0
          Fidget_Timer = Current_Time
          PlayerFidgeting = 0
          Fidget_Interval = 5000
          Fidget_Speed = 250
          PlayerSleeping = 0
          PlayerFiring = 0
          LaserSpeed = 8
          RobotArmorHit = 5
          PlayerArmorHit = 10
          AllowableDistance = 30  
          RHits = 5
          
          AddRobot(10,1,2,21,TileWidth,TileHeight)
          AddRobot(16,1,2,21,TileWidth,TileHeight)
          AddRobot(9,4,1,10,TileWidth,TileHeight)
          AddRobot(12,4,12,21,TileWidth,TileHeight)
          AddRobot(8,9,1,8,TileWidth,TileHeight)
          AddRobot(16,9,14,21,TileWidth,TileHeight)
          AddRobot(28,12,20,28,TileWidth,TileHeight)
          AddRobot(3,13,1,10,TileWidth,TileHeight)
          AddRobot(16,13,12,18,TileWidth,TileHeight)
          AddRobot(5,17,4,26,TileWidth,TileHeight)
          AddRobot(14,17,4,26,TileWidth,TileHeight)
          AddRobot(19,17,4,26,TileWidth,TileHeight)
          AddRobot(4,20,1,24,TileWidth,TileHeight)
          AddRobot(7,20,1,21,TileWidth,TileHeight)
          AddRobot(14,20,1,21,TileWidth,TileHeight)
          AddRobot(21,20,1,21,TileWidth,TileHeight)
          AddRobot(3,24,1,13,TileWidth,TileHeight)
          AddRobot(11,24,1,13,TileWidth,TileHeight)
          AddRobot(16,24,16,25,TileWidth,TileHeight)
          AddRobot(1,28,1,26,TileWidth,TileHeight)
          AddRobot(15,28,1,26,TileWidth,TileHeight)
          AddRobot(23,28,1,26,TileWidth,TileHeight)
          AddRobot(3,37,1,26,TileWidth,TileHeight)
          AddRobot(7,37,1,26,TileWidth,TileHeight)
          AddRobot(13,37,1,26,TileWidth,TileHeight)
          AddRobot(17,37,1,26,TileWidth,TileHeight)
          AddRobot(21,37,1,26,TileWidth,TileHeight)

          AddHealthPak(3,1,TileWidth,TileHeight)
          AddHealthPak(19,1,TileWidth,TileHeight)
          AddHealthPak(10,4,TileWidth,TileHeight)
          AddHealthPak(14,4,TileWidth,TileHeight)
          AddHealthPak(8,9,TileWidth,TileHeight)
          AddHealthPak(14,9,TileWidth,TileHeight)
          AddHealthPak(24,12,TileWidth,TileHeight)
          AddHealthPak(4,13,TileWidth,TileHeight)
          AddHealthPak(17,13,TileWidth,TileHeight)
          AddHealthPak(24,17,TileWidth,TileHeight)
          AddHealthPak(16,17,TileWidth,TileHeight)
          AddHealthPak(5,17,TileWidth,TileHeight)
          AddHealthPak(8,20,TileWidth,TileHeight)
          AddHealthPak(27,20,TileWidth,TileHeight)
          AddHealthPak(28,20,TileWidth,TileHeight)
          AddHealthPak(23,20,TileWidth,TileHeight)
          AddHealthPak(13,20,TileWidth,TileHeight)
          AddHealthPak(1,20,TileWidth,TileHeight)
          AddHealthPak(1,24,TileWidth,TileHeight)
          AddHealthPak(12,24,TileWidth,TileHeight)
          AddHealthPak(18,24,TileWidth,TileHeight)
          AddHealthPak(26,24,TileWidth,TileHeight)
          AddHealthPak(3,28,TileWidth,TileHeight)
          AddHealthPak(14,28,TileWidth,TileHeight)
          AddHealthPak(26,28,TileWidth,TileHeight)
          AddHealthPak(7,37,TileWidth,TileHeight)
          AddHealthPak(11,37,TileWidth,TileHeight)
          AddHealthPak(18,37,TileWidth,TileHeight)
          AddHealthPak(22,37,TileWidth,TileHeight)
          AddHealthPak(28,37,TileWidth,TileHeight)

  EndSelect

  ; loop until the player releases the space key so we're not firing on level launch
  Repeat
        ExamineKeyboard()
  Until KeyboardPushed(#PB_Key_Space) = 0
  
EndProcedure
; IDE Options = PureBasic 4.61 (Windows - x64)
; CursorPosition = 113
; FirstLine = 88
; Folding = -